# pyright: reportPrivateUsage=false
from pathlib import Path

import pytest
from pytest import LogCaptureFixture

from maritime_schema.api import MaritimeSchemaProcess, run


def test_file_not_found_exception():
    # Prepare
    config_file = Path("this_file_does_not_exist")
    # Execute and Assert
    with pytest.raises(FileNotFoundError):
        _ = run(config_file)


def test_run():
    # Prepare
    config_file = Path("test_config_file")
    # Execute
    run(config_file=config_file)
    # Assert
    # (nothin to assert. Assertion is that no exception is thrown.)


def test_run_with_option(caplog: LogCaptureFixture):
    # Prepare
    config_file = Path("test_config_file")
    log_level_expected = "INFO"
    log_message_expected = "option is True. maritime-schema process will do something differently."
    caplog.clear()
    # Execute
    run(config_file=config_file, option=True)
    # Assert
    assert len(caplog.records) > 0
    assert caplog.records[0].levelname == log_level_expected
    assert caplog.records[0].message == log_message_expected


class TestMaritimeSchemaProcess:
    def test_init(self):
        # Prepare
        config_file = Path("test_config_file.json")
        # Execute
        process = MaritimeSchemaProcess(config_file=config_file)
        # Assert
        assert process.config_file is config_file
        assert process.max_number_of_runs == 3
        assert process.run_number == 0
        assert process.terminate is False

    def test_init_with_empty_config_file(self):
        # sourcery skip: class-extract-method
        # Prepare
        config_file = Path("test_config_file_empty.json")
        # Execute
        process = MaritimeSchemaProcess(config_file=config_file)
        # Assert
        assert process.config_file is config_file
        assert process.max_number_of_runs == 1
        assert process.run_number == 0
        assert process.terminate is False


# @TODO: To be implemented
@pytest.mark.skip(reason="To be implemented")
def test_example_skip():
    """Example of a test skipped because it is not yet implemented."""
    pass



import logging
import os
from pathlib import Path
from typing import Union

from dictIO import DictReader

from maritime_schema.subpackage1.module1 import int_to_str
from maritime_schema.subpackage2.module2 import str_to_int

__ALL__ = ["run", "MaritimeSchemaProcess"]

logger = logging.getLogger(__name__)


def run(
    config_file: Union[str, os.PathLike[str]],
    option: bool = False,
):
    """Run the maritime-schema process.

    Run the maritime-schema process and .. (long description).

    Parameters
    ----------
    config_file : Union[str, os.PathLike[str]]
        file containing the maritime-schema configuration
    option : bool, optional
        if True, does something differently, by default False

    Raises
    ------
    FileNotFoundError
        if config_file does not exist
    """

    # Make sure config_file argument is of type Path. If not, cast it to Path type.
    config_file = config_file if isinstance(config_file, Path) else Path(config_file)

    # Check whether config file exists
    if not config_file.exists():
        logger.error(f"run: File {config_file} not found.")
        raise FileNotFoundError(config_file)

    if option:
        logger.info("option is True. maritime-schema process will do something differently.")

    process = MaritimeSchemaProcess(config_file)
    process.run()

    return


class MaritimeSchemaProcess:
    """Top level class encapsulating the maritime-schema process."""

    def __init__(
        self,
        config_file: Path,
    ):
        self.config_file: Path = config_file
        self._run_number: int = 0
        self._max_number_of_runs: int = 1
        self.terminate: bool = False
        self._read_config_file()
        return

    def run(self):
        """Run the maritime-schema process.

        Runs the maritime-schema process in a self-terminated loop.
        """

        # Run maritime-schema process until termination is flagged
        while not self.terminate:
            self._run_process()
            self.terminate = self._run_number >= self._max_number_of_runs
        return

    @property
    def run_number(self) -> int:
        """Example for a read only property."""
        return self._run_number

    @property
    def max_number_of_runs(self) -> int:
        """Example for a read/write property implemented through a pair of explicit
        getter and setter methods (see below for the related setter method).
        """
        return self._max_number_of_runs

    @max_number_of_runs.setter
    def max_number_of_runs(self, value: int):
        """Setter method that belongs to above getter method.

        Note that implementing specific getter- and setter methods is in most cases not necessary.
        The same can be achieved by simply making the instance variable a public attribute.
        I.e., declaring the instance variable in __init__() not as
        self._max_number_of_runs: int = ..  # (-> private instance variable)
        but as
        self.max_number_of_runs: int = ..   # (-> public attribute)

        However, in some cases the additional effort of implementing explicit getter- and setter- methods
        as in this example can be reasoned, for instance if you have a need for increased control
        and want be able to cancel or alter code execution, or write log messages whenever a property
        gets reads or written from outside.
        """

        self._max_number_of_runs = value
        return

    def _run_process(self):
        """Execute a single run of the maritime-schema process."""
        self._run_number += 1

        logger.info(f"Start run {self._run_number}")

        # Do stuff
        _string: str = _do_cool_stuff(self._run_number)
        _number: int = _do_even_cooler_stuff(_string)
        logger.debug(f"\t _string: {_string} _number: {_number}")

        logger.info(f"Successfully finished run {self._run_number}")

        return

    def _read_config_file(self):
        """Read config file."""
        config = DictReader.read(self.config_file)
        if "max_number_of_runs" in config:
            self._max_number_of_runs = config["max_number_of_runs"]
        return


def _do_cool_stuff(run_number: int) -> str:
    """Do cool stuff.

    Converts the passed in run number to a string.

    Parameters
    ----------
    run_number : int
        the run number

    Returns
    -------
    str
        the run number converted to string
    """
    result: str = int_to_str(run_number)
    return result


def _do_even_cooler_stuff(string: str) -> int:
    """Do even cooler stuff.

    Converts the passed in string-formatted integer back to an integer.

    Parameters
    ----------
    string : str
        the string-formatted integer to be converted back to an integer

    Returns
    -------
    int
        the resulting integer
    """
    result: int = str_to_int(string)
    return result



import logging

__ALL__ = ["int_to_str"]

logger = logging.getLogger(__name__)


def int_to_str(input: int) -> str:
    """Convert integer to string.

    This function converts an integer to its string representation.

    Parameters
    ----------
    input : int
        the integer to be converted to string

    Returns
    -------
    str
        the resulting string
    """
    logger.debug(f"function int_to_str() in subpackage1 called with argument {input}")
    return str(input)



import logging

__ALL__ = ["str_to_int"]

logger = logging.getLogger(__name__)


def str_to_int(input: str) -> int:
    """Convert string to integer.

    This function converts a string to an integer.
    The passed in string must represent a valid integer value.

    Parameters
    ----------
    input : str
        the string literal to be converted to integer

    Returns
    -------
    int
        the resulting integer

    Raises
    ------
    ValueError
        if conversion to integer is not possible, i.e. the string literal does not represent a valid integer number.
    """
    logger.debug(f"function str_to_int() in subpackage2 called with argument {input}")
    result: int
    try:
        result = int(input)
    except ValueError as e:
        logger.exception(f"ValueError raised in function str_to_int() in subpackage2. input was: {input}")
        raise ValueError from e
    return result
